# Rich Text

This module contains helper functions to convert HTML or a DOM tree into a rich text value and back, and to modify the value with functions that are similar to `String` methods, plus some additional ones for formatting.

## Installation

Install the module

```bash
npm install @wordpress/rich-text
```

_This package assumes that your code will run in an **ES2015+** environment. If you're using an environment that has limited or no support for ES2015+ such as lower versions of IE then using [core-js](https://github.com/zloirock/core-js) or [@babel/polyfill](https://babeljs.io/docs/en/next/babel-polyfill) will add support for these methods. Learn more about it in [Babel docs](https://babeljs.io/docs/en/next/caveats)._

## API

<!-- START TOKEN(Autogenerated API docs) -->

<a name="applyFormat" href="#applyFormat">#</a> **applyFormat**

Apply a format object to a Rich Text value from the given `startIndex` to the
given `endIndex`. Indices are retrieved from the selection if none are
provided.

_Parameters_

-   _value_ `Object`: Value to modify.
-   _format_ `Object`: Format to apply.
-   _startIndex_ `[number]`: Start index.
-   _endIndex_ `[number]`: End index.

_Returns_

-   `Object`: A new value with the format applied.

<a name="concat" href="#concat">#</a> **concat**

Combine all Rich Text values into one. This is similar to
`String.prototype.concat`.

_Parameters_

-   _values_ `...Object`: Objects to combine.

_Returns_

-   `Object`: A new value combining all given records.

<a name="create" href="#create">#</a> **create**

Create a RichText value from an `Element` tree (DOM), an HTML string or a
plain text string, with optionally a `Range` object to set the selection. If
called without any input, an empty value will be created. If
`multilineTag` is provided, any content of direct children whose type matches
`multilineTag` will be separated by two newlines. The optional functions can
be used to filter out content.

A value will have the following shape, which you are strongly encouraged not
to modify without the use of helper functions:

```js
{
  text: string,
  formats: Array,
  replacements: Array,
  ?start: number,
  ?end: number,
}
```

As you can see, text and formatting are separated. `text` holds the text,
including any replacement characters for objects and lines. `formats`,
`objects` and `lines` are all sparse arrays of the same length as `text`. It
holds information about the formatting at the relevant text indices. Finally
`start` and `end` state which text indices are selected. They are only
provided if a `Range` was given.

_Parameters_

-   _$1_ `[Object]`: Optional named arguments.
-   _$1.element_ `[Element]`: Element to create value from.
-   _$1.text_ `[string]`: Text to create value from.
-   _$1.html_ `[string]`: HTML to create value from.
-   _$1.range_ `[Range]`: Range to create value from.
-   _$1.multilineTag_ `[string]`: Multiline tag if the structure is multiline.
-   _$1.multilineWrapperTags_ `[Array]`: Tags where lines can be found if nesting is possible.
-   _$1.preserveWhiteSpace_ `[?boolean]`: Whether or not to collapse white space characters.

_Returns_

-   `Object`: A rich text value.

<a name="getActiveFormat" href="#getActiveFormat">#</a> **getActiveFormat**

Gets the format object by type at the start of the selection. This can be
used to get e.g. the URL of a link format at the current selection, but also
to check if a format is active at the selection. Returns undefined if there
is no format at the selection.

_Parameters_

-   _value_ `Object`: Value to inspect.
-   _formatType_ `string`: Format type to look for.

_Returns_

-   `(Object|undefined)`: Active format object of the specified type, or undefined.

<a name="getActiveObject" href="#getActiveObject">#</a> **getActiveObject**

Gets the active object, if there is any.

_Parameters_

-   _value_ `Object`: Value to inspect.

_Returns_

-   `?Object`: Active object, or undefined.

<a name="getTextContent" href="#getTextContent">#</a> **getTextContent**

Get the textual content of a Rich Text value. This is similar to
`Element.textContent`.

_Parameters_

-   _value_ `Object`: Value to use.

_Returns_

-   `string`: The text content.

<a name="insert" href="#insert">#</a> **insert**

Insert a Rich Text value, an HTML string, or a plain text string, into a
Rich Text value at the given `startIndex`. Any content between `startIndex`
and `endIndex` will be removed. Indices are retrieved from the selection if
none are provided.

_Parameters_

-   _value_ `Object`: Value to modify.
-   _valueToInsert_ `(Object|string)`: Value to insert.
-   _startIndex_ `[number]`: Start index.
-   _endIndex_ `[number]`: End index.

_Returns_

-   `Object`: A new value with the value inserted.

<a name="insertObject" href="#insertObject">#</a> **insertObject**

Insert a format as an object into a Rich Text value at the given
`startIndex`. Any content between `startIndex` and `endIndex` will be
removed. Indices are retrieved from the selection if none are provided.

_Parameters_

-   _value_ `Object`: Value to modify.
-   _formatToInsert_ `Object`: Format to insert as object.
-   _startIndex_ `[number]`: Start index.
-   _endIndex_ `[number]`: End index.

_Returns_

-   `Object`: A new value with the object inserted.

<a name="isCollapsed" href="#isCollapsed">#</a> **isCollapsed**

Check if the selection of a Rich Text value is collapsed or not. Collapsed
means that no characters are selected, but there is a caret present. If there
is no selection, `undefined` will be returned. This is similar to
`window.getSelection().isCollapsed()`.

_Parameters_

-   _value_ `Object`: The rich text value to check.

_Returns_

-   `(boolean|undefined)`: True if the selection is collapsed, false if not, undefined if there is no selection.

<a name="isEmpty" href="#isEmpty">#</a> **isEmpty**

Check if a Rich Text value is Empty, meaning it contains no text or any
objects (such as images).

_Parameters_

-   _value_ `Object`: Value to use.

_Returns_

-   `boolean`: True if the value is empty, false if not.

<a name="join" href="#join">#</a> **join**

Combine an array of Rich Text values into one, optionally separated by
`separator`, which can be a Rich Text value, HTML string, or plain text
string. This is similar to `Array.prototype.join`.

_Parameters_

-   _values_ `Array<Object>`: An array of values to join.
-   _separator_ `[(string|Object)]`: Separator string or value.

_Returns_

-   `Object`: A new combined value.

<a name="registerFormatType" href="#registerFormatType">#</a> **registerFormatType**

Registers a new format provided a unique name and an object defining its
behavior.

_Parameters_

-   _name_ `string`: Format name.
-   _settings_ `WPFormat`: Format settings.

_Returns_

-   `(WPFormat|undefined)`: The format, if it has been successfully registered; otherwise `undefined`.

<a name="remove" href="#remove">#</a> **remove**

Remove content from a Rich Text value between the given `startIndex` and
`endIndex`. Indices are retrieved from the selection if none are provided.

_Parameters_

-   _value_ `Object`: Value to modify.
-   _startIndex_ `[number]`: Start index.
-   _endIndex_ `[number]`: End index.

_Returns_

-   `Object`: A new value with the content removed.

<a name="removeFormat" href="#removeFormat">#</a> **removeFormat**

Remove any format object from a Rich Text value by type from the given
`startIndex` to the given `endIndex`. Indices are retrieved from the
selection if none are provided.

_Parameters_

-   _value_ `Object`: Value to modify.
-   _formatType_ `string`: Format type to remove.
-   _startIndex_ `[number]`: Start index.
-   _endIndex_ `[number]`: End index.

_Returns_

-   `Object`: A new value with the format applied.

<a name="replace" href="#replace">#</a> **replace**

Search a Rich Text value and replace the match(es) with `replacement`. This
is similar to `String.prototype.replace`.

_Parameters_

-   _value_ `Object`: The value to modify.
-   _pattern_ `(RegExp|string)`: A RegExp object or literal. Can also be a string. It is treated as a verbatim string and is not interpreted as a regular expression. Only the first occurrence will be replaced.
-   _replacement_ `(Function|string)`: The match or matches are replaced with the specified or the value returned by the specified function.

_Returns_

-   `Object`: A new value with replacements applied.

<a name="slice" href="#slice">#</a> **slice**

Slice a Rich Text value from `startIndex` to `endIndex`. Indices are
retrieved from the selection if none are provided. This is similar to
`String.prototype.slice`.

_Parameters_

-   _value_ `Object`: Value to modify.
-   _startIndex_ `[number]`: Start index.
-   _endIndex_ `[number]`: End index.

_Returns_

-   `Object`: A new extracted value.

<a name="split" href="#split">#</a> **split**

Split a Rich Text value in two at the given `startIndex` and `endIndex`, or
split at the given separator. This is similar to `String.prototype.split`.
Indices are retrieved from the selection if none are provided.

_Parameters_

-   _value_ `Object`: 
-   _value.formats_ `Array<Object>`: 
-   _value.replacements_ `Array<Object>`: 
-   _value.text_ `string`: 
-   _value.start_ `number`: 
-   _value.end_ `number`: 
-   _string_ `[(number|string)]`: Start index, or string at which to split.

_Returns_

-   `Array`: An array of new values.

<a name="toggleFormat" href="#toggleFormat">#</a> **toggleFormat**

Toggles a format object to a Rich Text value at the current selection.

_Parameters_

-   _value_ `Object`: Value to modify.
-   _format_ `Object`: Format to apply or remove.

_Returns_

-   `Object`: A new value with the format applied or removed.

<a name="toHTMLString" href="#toHTMLString">#</a> **toHTMLString**

Create an HTML string from a Rich Text value. If a `multilineTag` is
provided, text separated by a line separator will be wrapped in it.

_Parameters_

-   _$1_ `Object`: Named argements.
-   _$1.value_ `Object`: Rich text value.
-   _$1.multilineTag_ `[string]`: Multiline tag.
-   _$1.preserveWhiteSpace_ `[?boolean]`: Whether or not to use newline characters for line breaks.

_Returns_

-   `string`: HTML string.

<a name="unregisterFormatType" href="#unregisterFormatType">#</a> **unregisterFormatType**

Unregisters a format.

_Parameters_

-   _name_ `string`: Format name.

_Returns_

-   `(WPFormat|undefined)`: The previous format value, if it has been successfully unregistered; otherwise `undefined`.


<!-- END TOKEN(Autogenerated API docs) -->

<br/><br/><p align="center"><img src="https://s.w.org/style/images/codeispoetry.png?1" alt="Code is Poetry." /></p>
